import * as tapable from "tapable";
import { Source } from "webpack-sources";
import { JsCompilation, JsAssetInfo } from "@rspack/binding";
import { RspackOptionsNormalized } from "./config";
import { ResolvedOutput } from "./config/output";
import { ChunkGroup } from "./chunk_group";
import { Compiler } from "./compiler";
import ResolverFactory from "./ResolverFactory";
export declare type AssetInfo = Partial<JsAssetInfo> & Record<string, any>;
export declare class Compilation {
    #private;
    hooks: {
        processAssets: tapable.AsyncSeriesHook<Record<string, Source>>;
    };
    options: RspackOptionsNormalized;
    outputOptions: ResolvedOutput;
    compiler: Compiler;
    resolverFactory: ResolverFactory;
    constructor(compiler: Compiler, inner: JsCompilation);
    get hash(): string;
    get fullHash(): string;
    /**
     * Get a map of all assets.
     *
     * Source: [assets](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L1008-L1009)
     */
    get assets(): Record<string, Source>;
    /**
     * Get a map of all entrypoints.
     */
    get entrypoints(): Map<string, ChunkGroup>;
    /**
     * Update an existing asset. Trying to update an asset that doesn't exist will throw an error.
     *
     * See: [Compilation.updateAsset](https://webpack.js.org/api/compilation-object/#updateasset)
     * Source: [updateAsset](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4320)
     *
     * FIXME: *AssetInfo* may be undefined in update fn for webpack impl, but still not implemented in rspack
     *
     * @param {string} file file name
     * @param {Source | function(Source): Source} newSourceOrFunction new asset source or function converting old to new
     * @param {JsAssetInfo | function(JsAssetInfo): JsAssetInfo} assetInfoUpdateOrFunction new asset info or function converting old to new
     */
    updateAsset(filename: string, newSourceOrFunction: Source | ((source: Source) => Source), assetInfoUpdateOrFunction: JsAssetInfo | ((assetInfo: JsAssetInfo) => JsAssetInfo)): void;
    /**
     * Emit an not existing asset. Trying to emit an asset that already exists will throw an error.
     *
     * See: [Compilation.emitAsset](https://webpack.js.org/api/compilation-object/#emitasset)
     * Source: [emitAsset](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4239)
     *
     * @param {string} file file name
     * @param {Source} source asset source
     * @param {JsAssetInfo} assetInfo extra asset information
     * @returns {void}
     */
    emitAsset(filename: string, source: Source, assetInfo?: AssetInfo): void;
    deleteAsset(filename: string): void;
    /**
     * Get an array of Asset
     *
     * See: [Compilation.getAssets](https://webpack.js.org/api/compilation-object/#getassets)
     * Source: [getAssets](https://github.com/webpack/webpack/blob/9fcaa243573005d6fdece9a3f8d89a0e8b399613/lib/Compilation.js#L4448)
     *
     * @return {Readonly<JsAsset>[]}
     */
    getAssets(): {
        source: Source;
        name: string;
        info: JsAssetInfo;
    }[];
    getAsset(name: string): {
        source: Source;
        name: string;
        info: JsAssetInfo;
    };
    pushDiagnostic(severity: "error" | "warning", title: string, message: string): void;
    getPath(filename: string, data?: Record<string, any>): any;
    getAssetPath(filename: any, data: any): any;
    /**
     * Get the `Source` of an given asset filename.
     *
     * Note: This is not a webpack public API, maybe removed in future.
     *
     * @internal
     */
    __internal__getAssetSource(filename: string): Source | null;
    /**
     * Get a list of asset filenames.
     *
     * Note: This is not a webpack public API, maybe removed in future.
     *
     * @internal
     */
    __internal__getAssetFilenames(): string[];
    /**
     * Test if an asset exists.
     *
     * Note: This is not a webpack public API, maybe removed in future.
     *
     * @internal
     */
    __internal__hasAsset(name: string): boolean;
    createStats(): {};
    seal(): void;
    unseal(): void;
}
export type { JsAssetInfo };
//# sourceMappingURL=compilation.d.ts.map