"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveModuleOptions = exports.createRawModuleRuleUses = void 0;
const assert_1 = __importDefault(require("assert"));
const path_1 = __importDefault(require("path"));
const webpack_sources_1 = require("webpack-sources");
const util_1 = require("../util");
const createHash_1 = require("../util/createHash");
const identifier_1 = require("../util/identifier");
const memoize_1 = require("../util/memoize");
const devtool_1 = require("./devtool");
const resolve_1 = require("./resolve");
const toBuffer = (bufLike) => {
    if (Buffer.isBuffer(bufLike)) {
        return bufLike;
    }
    else if (typeof bufLike === "string") {
        return Buffer.from(bufLike);
    }
    throw new Error("Buffer or string expected");
};
function composeJsUse(uses, options, allUses) {
    if (!uses.length) {
        return null;
    }
    async function loader(data) {
        var _a, _b, _c, _d, _e, _f, _g;
        const compiler = options.getCompiler();
        const resolver = compiler.resolverFactory.get("normal");
        const moduleContext = path_1.default.dirname(data.resourcePath);
        let cacheable = data.cacheable;
        let buildDependencies = data.buildDependencies;
        let content = data.content;
        let sourceMap = (_a = data.sourceMap) === null || _a === void 0 ? void 0 : _a.toString("utf-8");
        let additionalData = data.additionalData
            ? JSON.parse(data.additionalData.toString("utf-8"))
            : undefined;
        // Loader is executed from right to left
        for (const use of uses) {
            (0, assert_1.default)("loader" in use);
            const loaderIndex = allUses.indexOf(use);
            let loaderResult;
            const p = new Promise((resolve, reject) => {
                var _a;
                let isDone = false;
                // Whether a `callback` or `async` is called
                let isSync = true;
                let isError = false; // internal error
                let reportedError = false;
                const fileDependencies = [];
                const contextDependencies = [];
                const missingDependencies = [];
                function callback(err, content, sourceMap, additionalData) {
                    if (isDone) {
                        if (reportedError)
                            return; // ignore
                        err = new Error("callback(): The callback was already called.");
                    }
                    isSync = false;
                    isDone = true;
                    if (err) {
                        isError = true;
                        reject(err);
                        return;
                    }
                    resolve({
                        cacheable,
                        content,
                        sourceMap,
                        additionalData,
                        fileDependencies,
                        contextDependencies,
                        missingDependencies
                    });
                }
                const getResolveContext = () => {
                    return {
                        fileDependencies: {
                            add: d => {
                                // FIXME: This will cause build hang, so we skip it
                            }
                        },
                        contextDependencies: {
                            add: d => loaderContext.addContextDependency(d)
                        },
                        missingDependencies: {
                            add: d => loaderContext.addMissingDependency(d)
                        }
                    };
                };
                const getAbsolutify = (0, memoize_1.memoize)(() => identifier_1.absolutify.bindCache(compiler.root));
                const getAbsolutifyInContext = (0, memoize_1.memoize)(() => identifier_1.absolutify.bindContextCache(moduleContext, compiler.root));
                const getContextify = (0, memoize_1.memoize)(() => identifier_1.contextify.bindCache(compiler.root));
                const getContextifyInContext = (0, memoize_1.memoize)(() => identifier_1.contextify.bindContextCache(moduleContext, compiler.root));
                const utils = {
                    absolutify: (context, request) => {
                        return context === moduleContext
                            ? getAbsolutifyInContext()(request)
                            : getAbsolutify()(context, request);
                    },
                    contextify: (context, request) => {
                        return context === moduleContext
                            ? getContextifyInContext()(request)
                            : getContextify()(context, request);
                    },
                    createHash: type => {
                        return (0, createHash_1.createHash)(type || compiler.compilation.outputOptions.hashFunction);
                    }
                };
                const loaderContext = {
                    version: 2,
                    sourceMap: (0, devtool_1.isUseSourceMap)(options.devtool),
                    resourcePath: data.resourcePath,
                    resource: data.resource,
                    // Return an empty string if there is no query or fragment
                    resourceQuery: data.resourceQuery || "",
                    resourceFragment: data.resourceFragment || "",
                    loaderIndex,
                    mode: compiler.options.mode,
                    hot: (_a = compiler.options.devServer) === null || _a === void 0 ? void 0 : _a.hot,
                    getOptions(schema) {
                        let { options } = use;
                        if (options === null || options === undefined) {
                            options = {};
                        }
                        if (schema) {
                            let name = "Loader";
                            let baseDataPath = "options";
                            let match;
                            if (schema.title && (match = /^(.+) (.+)$/.exec(schema.title))) {
                                [, name, baseDataPath] = match;
                            }
                            const { validate } = require("schema-utils");
                            validate(schema, options, {
                                name,
                                baseDataPath
                            });
                        }
                        return options;
                    },
                    get query() {
                        return use.options && typeof use.options === "object"
                            ? use.options
                            : use.query;
                    },
                    get data() {
                        return use.data;
                    },
                    resolve(context, request, callback) {
                        resolver.resolve({}, context, request, getResolveContext(), callback);
                    },
                    getResolve(options) {
                        const child = options ? resolver.withOptions(options) : resolver;
                        return (context, request, callback) => {
                            if (callback) {
                                child.resolve({}, context, request, getResolveContext(), callback);
                            }
                            else {
                                return new Promise((resolve, reject) => {
                                    child.resolve({}, context, request, getResolveContext(), (err, result) => {
                                        if (err)
                                            reject(err);
                                        else
                                            resolve(result);
                                    });
                                });
                            }
                        };
                    },
                    getLogger(name) {
                        return compiler.getInfrastructureLogger(() => [name, data.resource].filter(Boolean).join("|"));
                    },
                    cacheable(value) {
                        cacheable = value;
                    },
                    async() {
                        if (isDone) {
                            if (reportedError)
                                return; // ignore
                            reject(new Error("async(): The callback was already called."));
                        }
                        isSync = false;
                        return callback;
                    },
                    callback,
                    rootContext: options.context,
                    context: moduleContext,
                    emitError(error) {
                        const title = "Module Error";
                        const message = error instanceof Error
                            ? `${error.message}${error.stack ? `\n${error.stack}` : ""}`
                            : error;
                        compiler.compilation.pushDiagnostic("error", title, `${message}\n(from: ${use.loader})`);
                    },
                    emitWarning(warning) {
                        const title = "Module Warning";
                        const message = warning instanceof Error
                            ? `${warning.message}${warning.stack ? `\n${warning.stack}` : ""}`
                            : warning;
                        compiler.compilation.pushDiagnostic("warning", title, `${message}\n(from: ${use.loader})`);
                    },
                    emitFile(name, content, sourceMap, assetInfo) {
                        let source;
                        if (sourceMap) {
                            if (typeof sourceMap === "string" &&
                                (loaderContext.sourceMap ||
                                    (0, devtool_1.isUseSimpleSourceMap)(options.devtool))) {
                                source = new webpack_sources_1.OriginalSource(content, (0, identifier_1.makePathsRelative)(moduleContext, sourceMap, compiler));
                            }
                            if (this.useSourceMap) {
                                source = new webpack_sources_1.SourceMapSource(content, // webpack-sources type declaration is wrong
                                name, (0, identifier_1.makePathsRelative)(moduleContext, sourceMap, compiler) // webpack-sources type declaration is wrong
                                );
                            }
                        }
                        else {
                            source = new webpack_sources_1.RawSource(content); // webpack-sources type declaration is wrong
                        }
                        compiler.compilation.emitAsset(name, source, assetInfo);
                    },
                    fs: compiler.inputFileSystem,
                    utils,
                    addBuildDependency(file) {
                        buildDependencies.push(file);
                    },
                    addDependency(file) {
                        fileDependencies.push(file);
                    },
                    addContextDependency(context) {
                        contextDependencies.push(context);
                    },
                    addMissingDependency(missing) {
                        missingDependencies.push(missing);
                    },
                    clearDependencies() {
                        fileDependencies.length = 0;
                        contextDependencies.length = 0;
                        missingDependencies.length = 0;
                    },
                    getDependencies() {
                        return fileDependencies.slice();
                    },
                    getContextDependencies() {
                        return contextDependencies.slice();
                    },
                    getMissingDependencies() {
                        return missingDependencies.slice();
                    }
                };
                /**
                 * support loader as string
                 */
                let loader;
                if (typeof use.loader === "string") {
                    try {
                        let loaderPath = require.resolve(use.loader, {
                            paths: [options.context]
                        });
                        loader = require(loaderPath);
                    }
                    catch (err) {
                        reject(err);
                        return;
                    }
                }
                else {
                    loader = use.loader;
                }
                let result = undefined;
                try {
                    result = loader.apply(loaderContext, [
                        loader.raw ? Buffer.from(content) : content.toString("utf-8"),
                        sourceMap,
                        additionalData
                    ]);
                    if (isSync) {
                        isDone = true;
                        if (result === undefined) {
                            resolve({
                                content,
                                buildDependencies,
                                sourceMap,
                                additionalData,
                                cacheable,
                                fileDependencies,
                                contextDependencies,
                                missingDependencies
                            });
                            return;
                        }
                        if ((0, util_1.isPromiseLike)(result)) {
                            return result.then(function (result) {
                                resolve({
                                    content: result,
                                    buildDependencies,
                                    sourceMap,
                                    additionalData,
                                    cacheable,
                                    fileDependencies,
                                    contextDependencies,
                                    missingDependencies
                                });
                            }, reject);
                        }
                        return resolve({
                            content: result,
                            buildDependencies,
                            sourceMap,
                            additionalData,
                            cacheable,
                            fileDependencies,
                            contextDependencies,
                            missingDependencies
                        });
                    }
                }
                catch (err) {
                    if (isError) {
                        reject(err);
                        return;
                    }
                    if (isDone) {
                        // loader is already "done", so we cannot use the callback function
                        // for better debugging we print the error on the console
                        if (typeof err === "object" && err.stack)
                            console.error(err.stack);
                        else
                            console.error(err);
                        reject(err);
                        return;
                    }
                    isDone = true;
                    reportedError = true;
                    reject(err);
                }
            });
            if ((loaderResult = await p)) {
                additionalData =
                    (_b = (typeof loaderResult.additionalData === "string"
                        ? JSON.parse(loaderResult.additionalData)
                        : loaderResult.additionalData)) !== null && _b !== void 0 ? _b : additionalData;
                buildDependencies = (_c = loaderResult.buildDependencies) !== null && _c !== void 0 ? _c : buildDependencies;
                content = (_d = loaderResult.content) !== null && _d !== void 0 ? _d : content;
                sourceMap = (_e = loaderResult.sourceMap) !== null && _e !== void 0 ? _e : sourceMap;
                cacheable = (_f = loaderResult.cacheable) !== null && _f !== void 0 ? _f : cacheable;
                (_g = compiler.watcher) === null || _g === void 0 ? void 0 : _g.add([
                    data.resourcePath,
                    ...loaderResult.fileDependencies,
                    ...loaderResult.contextDependencies,
                    ...loaderResult.missingDependencies
                ]);
            }
        }
        return {
            cacheable: cacheable,
            buildDependencies: buildDependencies,
            content: toBuffer(content),
            sourceMap: sourceMap
                ? toBuffer(typeof sourceMap === "string"
                    ? sourceMap
                    : JSON.stringify(sourceMap))
                : undefined,
            additionalData: additionalData
                ? toBuffer(JSON.stringify(additionalData))
                : undefined
        };
    }
    loader.displayName = `NodeLoaderAdapter(${uses
        .map(item => {
        (0, assert_1.default)("loader" in item);
        let loader;
        if (typeof item.loader === "string") {
            try {
                const path = require.resolve(item.loader, {
                    paths: [options.context]
                });
                loader = require(path);
            }
            catch (e) {
                loader = null;
            }
        }
        else {
            loader = item.loader;
        }
        return (loader === null || loader === void 0 ? void 0 : loader.displayName) || (loader === null || loader === void 0 ? void 0 : loader.name) || "unknown-loader";
    })
        .join(" -> ")})`;
    return {
        loader
    };
}
function createRawModuleRuleUses(uses, options) {
    const allUses = [...uses].reverse();
    return createRawModuleRuleUsesImpl(allUses, options, allUses);
}
exports.createRawModuleRuleUses = createRawModuleRuleUses;
function createRawModuleRuleUsesImpl(uses, options, allUses) {
    if (!uses.length) {
        return [];
    }
    const index = uses.findIndex(use => "builtinLoader" in use);
    if (index < 0) {
        return [composeJsUse(uses, options, allUses)];
    }
    const before = uses.slice(0, index);
    const after = uses.slice(index + 1);
    return [
        composeJsUse(before, options, allUses),
        createNativeUse(uses[index]),
        ...createRawModuleRuleUsesImpl(after, options, allUses)
    ].filter((item) => Boolean(item));
}
function createNativeUse(use) {
    var _a;
    (0, assert_1.default)("builtinLoader" in use);
    if (use.builtinLoader === "sass-loader") {
        ((_a = use.options) !== null && _a !== void 0 ? _a : (use.options = {})).__exePath = require.resolve(`@tmp-sass-embedded/${process.platform}-${process.arch}/dart-sass-embedded/dart-sass-embedded${process.platform === "win32" ? ".bat" : ""}`);
    }
    return {
        builtinLoader: use.builtinLoader,
        options: JSON.stringify(use.options)
    };
}
function resolveModuleRuleCondition(condition) {
    if (typeof condition === "string") {
        return {
            type: "string",
            matcher: condition
        };
    }
    if (condition instanceof RegExp) {
        return {
            type: "regexp",
            matcher: condition.source
        };
    }
    throw new Error(`Unsupported condition type ${typeof condition}, value: ${condition}`);
}
function resolveModuleRuleConditions(conditions) {
    return conditions.map(resolveModuleRuleCondition);
}
function resolveModuleOptions(module = {}, options) {
    var _a;
    const rules = ((_a = module.rules) !== null && _a !== void 0 ? _a : []).map(rule => {
        // FIXME: use error handler instead of throwing
        if (rule === null || rule === void 0 ? void 0 : rule.loader) {
            throw new Error("`Rule.loader` is not supported, use `Rule.use` instead");
        }
        if (rule === null || rule === void 0 ? void 0 : rule.uses) {
            throw new Error("`Rule.uses` is deprecated for aligning with webpack, use `Rule.use` instead");
        }
        return {
            ...rule,
            test: (0, util_1.isNil)(rule.test) ? null : resolveModuleRuleCondition(rule.test),
            include: (0, util_1.isNil)(rule.include)
                ? null
                : Array.isArray(rule.include)
                    ? resolveModuleRuleConditions(rule.include)
                    : [resolveModuleRuleCondition(rule.include)],
            exclude: (0, util_1.isNil)(rule.exclude)
                ? null
                : Array.isArray(rule.exclude)
                    ? resolveModuleRuleConditions(rule.exclude)
                    : [resolveModuleRuleCondition(rule.exclude)],
            resource: (0, util_1.isNil)(rule.resource)
                ? null
                : resolveModuleRuleCondition(rule.resource),
            resourceQuery: (0, util_1.isNil)(rule.resourceQuery)
                ? null
                : resolveModuleRuleCondition(rule.resourceQuery),
            use: createRawModuleRuleUses(rule.use || [], options),
            resolve: (0, util_1.isNil)(rule.resolve)
                ? null
                : (0, resolve_1.resolveResolveOptions)(rule.resolve, options)
        };
    });
    return {
        parser: module.parser,
        rules
    };
}
exports.resolveModuleOptions = resolveModuleOptions;
//# sourceMappingURL=module.js.map